module RandomGif (..) where

-- From section 5 of the Elm Architecture Tutorial https://github.com/evancz/elm-architecture-tutorial#example-5-random-gif-viewer

import Testable.Effects as Effects exposing (Effects, Never)
import Html exposing (..)
import Html.Attributes exposing (style)
import Html.Events exposing (onClick)
import Testable.Http as Http
import Json.Decode as Json
import Testable.Task as Task


-- MODEL


type alias Model =
  { apiKey : String
  , topic : String
  , gifUrl : String
  }


init : String -> String -> ( Model, Effects Action )
init apiKey topic =
  ( Model apiKey topic "/favicon.ico"
  , getRandomGif apiKey topic
  )



-- UPDATE


type Action
  = RequestMore
  | NewGif (Maybe String)


update : Action -> Model -> ( Model, Effects Action )
update action model =
  case action of
    RequestMore ->
      ( model, getRandomGif model.apiKey model.topic )

    NewGif maybeUrl ->
      ( Model model.apiKey model.topic (Maybe.withDefault model.gifUrl maybeUrl)
      , Effects.none
      )



-- VIEW


(=>) =
  (,)


view : Signal.Address Action -> Model -> Html
view address model =
  div
    [ style [ "width" => "200px" ] ]
    [ h2 [ headerStyle ] [ text model.topic ]
    , div [ imgStyle model.gifUrl ] []
    , button [ onClick address RequestMore ] [ text "More Please!" ]
    ]


headerStyle : Attribute
headerStyle =
  style
    [ "width" => "200px"
    , "text-align" => "center"
    ]


imgStyle : String -> Attribute
imgStyle url =
  style
    [ "display" => "inline-block"
    , "width" => "200px"
    , "height" => "200px"
    , "background-position" => "center center"
    , "background-size" => "cover"
    , "background-image" => ("url('" ++ url ++ "')")
    ]



-- EFFECTS


getRandomGif : String -> String -> Effects Action
getRandomGif apiKey topic =
  Http.get decodeUrl (randomUrl apiKey topic)
    |> Task.toMaybe
    |> Task.map NewGif
    |> Effects.task


randomUrl : String -> String -> String
randomUrl apiKey topic =
  Http.url
    "https://api.giphy.com/v1/gifs/random"
    [ "api_key" => apiKey
    , "tag" => topic
    ]


decodeUrl : Json.Decoder String
decodeUrl =
  Json.at [ "data", "image_url" ] Json.string
